# encoding=UTF-8
from __future__ import unicode_literals

import decimal
import datetime
import warnings

import pytz

from .logging import logger


class ConfirmationPayload(object):
    RESPONSE_CODES = {
        0: 'Transacción aprobada.',
        -1: 'Rechazo de tx. en B24, No autorizada',
        -2: 'Transacción debe reintentarse.',
        -3: 'Error en tx.',
        -4: 'Rechazo de tx. En B24, No autorizada',
        -5: 'Rechazo por error de tasa.',
        -6: 'Excede cupo máximo mensual.',
        -7: 'Excede límite diario por transacción.',
        -8: 'Rubro no autorizado.'
    }
    SUCCESS_RESPONSE_CODE = 0
    PAYMENT_TYPES = {
        "VN": "Venta Normal",
        "VC": "Venta Cuotas",
        "SI": "Tres Cuotas Sin Interés",
        "S2": "Dos Cuotas Sin Interés",
        "CI": "Cuotas Comercio"
    }

    def __init__(self, data):
        self.data = data

    @property
    def paid_at(self):
        fecha_transaccion = self.data['TBK_FECHA_TRANSACCION']
        hora_transaccion = self.data['TBK_HORA_TRANSACCION']
        m = int(fecha_transaccion[:2])
        d = int(fecha_transaccion[2:])
        h = int(hora_transaccion[:2])
        i = int(hora_transaccion[2:4])
        s = int(hora_transaccion[4:])

        santiago = pytz.timezone('America/Santiago')
        today = santiago.localize(datetime.datetime.today())
        santiago_dt = santiago.localize(
            datetime.datetime(today.year, m, d, h, i, s))

        return santiago_dt

    @property
    def message(self):
        return self.RESPONSE_CODES[self.response]

    @property
    def amount(self):
        return decimal.Decimal(self.data['TBK_MONTO']) / 100

    @property
    def transaction_id(self):
        return int(self.data['TBK_ID_TRANSACCION'])

    @property
    def order_id(self):
        return self.data['TBK_ORDEN_COMPRA']

    @property
    def response(self):
        return int(self.data['TBK_RESPUESTA'])

    @property
    def credit_card_last_digits(self):
        return self.data['TBK_FINAL_NUMERO_TARJETA']

    @property
    def credit_card_number(self):
        return "XXXX-XXXX-XXXX-{last_digits}".format(last_digits=self.credit_card_last_digits)

    @property
    def authorization_code(self):
        return self.data['TBK_CODIGO_AUTORIZACION']

    @property
    def accountable_date(self):
        fecha_transaccion = self.data['TBK_FECHA_CONTABLE']
        m = int(fecha_transaccion[:2])
        d = int(fecha_transaccion[2:])
        santiago = pytz.timezone('America/Santiago')
        today = santiago.localize(datetime.datetime.today())
        year = today.year
        if self.paid_at.month == 12 and m == 1:
            year += 1
        santiago_dt = santiago.localize(datetime.datetime(year, m, d))
        return santiago_dt

    @property
    def session_id(self):
        session_id = self.data['TBK_ID_SESION']
        return session_id if session_id != 'null' else None

    @property
    def installments(self):
        return int(self.data['TBK_NUMERO_CUOTAS'])

    @property
    def payment_type(self):
        return self.PAYMENT_TYPES[self.payment_type_code]

    @property
    def payment_type_code(self):
        return self.data['TBK_TIPO_PAGO']

    def __getitem__(self, key):
        return self.data[key]


class Confirmation(object):

    def __init__(self, commerce, request_ip, data):
        self.commerce = commerce
        self.request_ip = request_ip
        self.payload = ConfirmationPayload(self.parse(data['TBK_PARAM']))
        logger.confirmation(self)

    def parse(self, tbk_param):
        decrypted_params, signature = self.commerce.webpay_decrypt(tbk_param)
        params = {}
        for line in decrypted_params.split('#'):
            index = line.find('=')
            params[line[:index]] = line[index + 1:]
        params['TBK_MAC'] = signature
        return params

    def is_success(self):
        return self.payload.response == self.payload.SUCCESS_RESPONSE_CODE

    @property
    def amount(self):
        return self.payload.amount

    @property
    def order_id(self):
        return self.payload.order_id

    @property
    def acknowledge(self):  # pragma: no cover
        warnings.warn(
            "Deprecated for commerce.acknowledge attribute.", DeprecationWarning)
        return self.commerce.acknowledge

    @property
    def reject(self):  # pragma: no cover
        warnings.warn(
            "Deprecated for commerce.reject attribute.", DeprecationWarning)
        return self.commerce.reject

    @property
    def message(self):  # pragma: no cover
        warnings.warn(
            "Deprecated for payload.message attribute.", DeprecationWarning)
        return self.payload.message

    @property
    def paid_at(self):  # pragma: no cover
        warnings.warn(
            "Deprecated for payload.paid_at attribute.", DeprecationWarning)
        return self.payload.paid_at

    @property
    def transaction_id(self):  # pragma: no cover
        warnings.warn(
            "Deprecated for payload.transaction_id attribute.", DeprecationWarning)
        return self.payload.transaction_id

    @property
    def params(self):  # pragma: no cover
        warnings.warn(
            "Deprecated for payload.data attribute.", DeprecationWarning)
        return self.payload.data



import os

from six.moves.urllib.parse import urlparse

from .encryption import Encryption, Decryption

from Crypto.PublicKey import RSA

__all__ = ['Commerce']


TEST_COMMERCE_KEY = """
-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEAn3HzPC1ZBzCO3edUCf/XJiwj3bzJpjjTi/zBO9O+DDzZCaMp
14aspxQryvJhv8644E19Q+NHfxtz1cxd2wnSYKvay1gJx30ZlTOAkzUj4QMimR16
vomLlQ3T2MAz1znt/PVPVU7T/JOG9R+EbiHNVKa/hUjwJEFVXLQNME97nHoLjb3v
V5yV2aVhmox7b54n6F3UVPHvCsHKbJpXpE+vnLpVmdETbNpFVrDygXyG+mnEvyiO
BLIwEY3XTMrgXvS069groLi5Gg8C5LDaYOWjE9084T4fiWGrHhn2781R1rykunTu
77wiWPuQHMS0+YC7mhnsk8Z/ilD+aWz/vhsgHwIDAQABAoIBAQCM+Nrt4cpNKQmn
+Ne8348CGRS9ACXp6WRg6OCQXO4zM7lRZAminVgZgSQXE6aJR+T9rIWMeG7GWydX
aJGzEEQJZOjV0MkUr+7mk9qiTOGkGHmGlyHnRQU8jDU59vXe3UEl3l5+NmwHbQht
waf9F7XLmoLK/WoVJA6tICRpCl1oQrpziqN+gjdmMpz9i8I1sMFE7+Y7xf+7S2u7
c1MRPUWqgdS9yViQVh3vZi25m5CyKRVnOB0hpNuZ7nrJymtADYSWt9wV2W1fX+MX
UUoYfxyQQvWryHhGdedU7GGAnoEdblUcDkBuAaFmsm1P8K4HQZLWP4v6pYlW2JLa
Zoaerb3BAoGBANCRevl0CLB0HBU7sCs0eN9fTkIEsh3OVIxPSBqDnKsynJrIWovK
cs37Vb6phzdQO3ADoFJvR9ck8+v6Cv0KR8IOFl9wfC4ZoxkKBBeq94ZLN+YhE2PW
KiRFybqcgCtzxKS3MyWgpIcT9xFtHVjlorZ8Jk51fgLZbGzamtLhderVAoGBAMO0
mIiiV4l2vXzu4tFfkpu/GOx/D9/vAic3X9FOky09BNCyuMXMQgI8e3wWsGEZghls
Vg9KDV5EPxAmpumcdPFK2IMACaH41ac7vys3ZD8kMK0INQkuDAcG4YsxMaTwEPo0
p1i3zwwEWwknw1yJkOyozz0EcIzS9NrZZEjnBHEjAoGAQ81XdeqzvHEyg/CQd6sq
NCtubGXMZYYi1C4d2Yi5kKn2YRcK4HDi23V+TWodK+0oNWToZIQKjbVUmn0Bv3rt
EvezbDlMFUx+SfCIng0VRJIFTQmpnQYNUxdg2gpwXC/ZWFa6CNxtQABMjFy1cqXM
PJild1IYseJurgBu3mkvBTUCgYBqA/T1X2woLUis2wPIBAv5juXDh3lkB6eU8uxX
CEe2I+3t2EM781B2wajrKadWkmjluMhN9AGV5UZ8S1P0DStUYwUywdx1/8RNmZIP
qSwHAGXV9jI0zNr7G4Em0/leriWkRM26w6fHjLx8EyxDfsohSbkqBrOptcWqoEUx
MOQ5HQKBgAS4sbddOas2MapuhKU2surEb3Kz3RCIpta4bXgTQMt9wawcZSSpvnfT
zs5sehYvBFszL3MV98Uc50HXMf7gykRCmPRmB9S+f+kiVRvQDHfc9nRNg2XgcotU
KAE16PQM8GihQ0C+EcXHouyud5CRJGfyurokRlH/jY3BiRAG5c+6
-----END RSA PRIVATE KEY-----
""".strip()

TEST_WEBPAY_KEY = """
-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAtKe3HHWwRcizAfkbS92V
fQr8cUb94TRjQPzNTqBduvvj65AD5J98Cn1htE3NzOz+PjPRcnfVe53V4f3+YlIb
6nnxyeuYLByiwoPkCmpOFBxNp04/Yh3dxN4xgOANXA37rNbDeO4WIEMG6zbdQMNJ
7RqQUlJSmui8gt3YxtqWBhBVW79qDCYVzxFrv3SH7pRuYEr+cxDvzRylxnJgr6ee
N7gmjoSMqF16f9aGdQ12obzV0A35BqpN6pRFoS/NvICbEeedS9g5gyUHf54a+juB
OV2HH5VJsCCgcb7I7Sio/xXTyP+QjIGJfpukkE8F+ohwRiChZ9jMXofPtuZYZiFQ
/gX08s5Qdpaph65UINP7crYbzpVJdrT2J0etyMcZbEanEkoX8YakLEBpPhyyR7mC
73fWd9sTuBEkG6kzCuG2JAyo6V8eyISnlKDEVd+/6G/Zpb5cUdBCERTYz5gvNoZN
zkuq4isiXh5MOLGs91H8ermuhdQe/lqvXf8Op/EYrAuxcdrZK0orI4LbPdUrC0Jc
Fl02qgXRrSpXo72anOlFc9P0blD4CMevW2+1wvIPA0DaJPsTnwBWOUqcfa7GAFH5
KGs3zCiZ5YTLDlnaps8koSssTVRi7LVT8HhiC5mjBklxmZjBv6ckgQeFWgp18kuU
ve5Elj5HSV7x2PCz8RKB4XcCAwEAAQ==
-----END PUBLIC KEY-----
""".strip()

WEBPAY_KEY = """
-----BEGIN PUBLIC KEY-----
MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAxKKjroxE7X44TQovh9A9
ZpntP7LrdoyFsnJbDKjOOCoiid92FydN5qemyQCeXhsc7QHUXwGdth22fB8xJr3a
MZBEUJ+BKFrL+W6yE5V+F5Bj0Uq3lL0QMAIftGhLpgqw0ZMtU89kyd9Q4Rclq4r8
p2m/ZD7Pn5EmTOFSeyoWTMZQDl7OEoCKh/cZH5NJdWL08lCI+sGLOghRmFzkve4h
F9JCwKA7NYG7j3BWh39Oj2NIXEY/TO1Y3Y2WfNv9nvTpr46SpFlyp0KOhSiqgvXX
DgeXlebyqS82ch2DzOV9fjDAw7t71WXJBAev8Gd6HXwIXE/JP6AnLCa2Y+b6Wv8K
GWBCMIBXWL0m7WHeCaJ9Hx2yXZmHJh8FgeKffFKCwn3X90JiMocOSGsOE+Sfo85S
h/39Vc7vZS3i7kJDDoz9ab9/vFy30RuJf4p8Erh7kWtERVoG6/EhR+j4N3mgIOBZ
SHfzDAoOnqP5l7t2RXYcEbRLVN6o+XgUtalX33EJxJRsXoz9a6PxYlesIwPbKteD
BZ/xyJDwTc2gU2YzSH8G9anKrcvITBDULSAuxQUkYOiLbkb7vSKWDYKe0do6ibO3
RY/KXI63Q7bGKYaI2aa/8GnqVJ2G1U2s59NpqX0aaWjn59gsA8trA0YKOZP4xJIh
CvLM94G4V7lxe2IHKPqLscMCAwEAAQ==
-----END PUBLIC KEY-----
""".strip()


class Commerce(object):
    TEST_COMMERCE_KEY = TEST_COMMERCE_KEY
    TEST_COMMERCE_ID = "597026007976"

    webpay_key_id = 101

    def __init__(self, id=None, key=None, testing=False):
        self.testing = testing
        self.id = self.__get_id(id)
        self.key = self.__get_key(key)

    @staticmethod
    def create_commerce():
        """
        Creates commerce from environment variables.
        """
        commerce_id = os.getenv('TBK_COMMERCE_ID')
        commerce_key = os.getenv('TBK_COMMERCE_KEY')
        commerce_testing = os.getenv('TBK_COMMERCE_TESTING') == 'True'

        if not commerce_testing:
            if commerce_id is None:
                raise ValueError("create_commerce needs TBK_COMMERCE_ID environment variable")
            if commerce_key is None:
                raise ValueError("create_commerce needs TBK_COMMERCE_KEY environment variable")

        return Commerce(
            id=commerce_id or Commerce.TEST_COMMERCE_ID,
            key=commerce_key,
            testing=commerce_testing
        )

    def __get_key(self, key):
        if not key:
            if self.testing:
                return self.TEST_COMMERCE_KEY
            raise TypeError("Commerce needs a key")
        return key

    def __get_id(self, id):
        if not id:
            if self.testing:
                return self.TEST_COMMERCE_ID
            raise TypeError("Commerce needs an id")
        return id

    def webpay_decrypt(self, encrypted):
        commerce_key = self.get_commerce_key()
        webpay_key = self.get_webpay_key()
        decryption = Decryption(commerce_key, webpay_key)
        decrypted = decryption.decrypt(encrypted)
        return decrypted

    def webpay_encrypt(self, decrypted):
        commerce_key = self.get_commerce_key()
        webpay_key = self.get_webpay_key()
        encryption = Encryption(commerce_key, webpay_key)
        return encryption.encrypt(decrypted)

    def get_webpay_key(self):
        return RSA.importKey(TEST_WEBPAY_KEY if self.testing else WEBPAY_KEY)

    def get_commerce_key(self):
        return RSA.importKey(self.key)

    def get_public_key(self):
        return self.get_commerce_key().publickey().exportKey()

    def get_config_tbk(self, confirmation_url):
        config = (
            "IDCOMERCIO = {commerce_id}\n"
            "MEDCOM = 1\n"
            "TBK_KEY_ID = 101\n"
            "PARAMVERIFCOM = 1\n"
            "URLCGICOM = {confirmation_path}\n"
            "SERVERCOM = {confirmation_host}\n"
            "PORTCOM = {confirmation_port}\n"
            "WHITELISTCOM = ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz 0123456789./:=&?_\n"
            "HOST = {confirmation_host}\n"
            "WPORT = {confirmation_port}\n"
            "URLCGITRA = /filtroUnificado/bp_revision.cgi\n"
            "URLCGIMEDTRA = /filtroUnificado/bp_validacion.cgi\n"
            "SERVERTRA = {webpay_server}\n"
            "PORTTRA = {webpay_port}\n"
            "PREFIJO_CONF_TR = HTML_\n"
            "HTML_TR_NORMAL = http://127.0.0.1/notify\n"
        )
        confirmation_uri = urlparse(confirmation_url)
        webpay_server = "https://certificacion.webpay.cl" if self.testing else "https://webpay.transbank.cl"
        webpay_port = 6443 if self.testing else 443
        return config.format(commerce_id=self.id,
                             confirmation_path=confirmation_uri.path,
                             confirmation_host=confirmation_uri.hostname,
                             confirmation_port=confirmation_uri.port,
                             webpay_port=webpay_port,
                             webpay_server=webpay_server)

    @property
    def acknowledge(self):
        return self.webpay_encrypt('ACK')

    @property
    def reject(self):
        return self.webpay_encrypt('ERR')



# encoding=UTF-8
from __future__ import unicode_literals

import os
import datetime
from contextlib import closing

import pytz


def event_payment_format(**kwargs):
    return PAYMENT_FORMAT.format(**kwargs)


def event_confirmation_format(**kwargs):
    return CONFIRMATION_FORMAT.format(**kwargs)


def log_confirmation_format(**kwargs):
    return JOURNAL_FORMAT % kwargs


EVENTS_LOG_FILE_NAME_FORMAT = "TBK_EVN%s.log"
EVENTS_LOG_FILE_DATE_FORMAT = "%Y%m%d"
JOURNAL_LOG_FILE_NAME_FORMAT = "tbk_bitacora_TR_NORMAL_%s.log"
JOURNAL_LOG_FILE_DATE_FORMAT = "%m%d"


class WebpayOfficialHandler(object):

    def __init__(self, path=None):
        self.path = path

    def event_payment(self, **kwargs):
        with closing(self.events_log_file) as events_log_file:
            events_log_file.write(event_payment_format(**kwargs))

    def event_confirmation(self, **kwargs):
        with closing(self.events_log_file) as events_log_file:
            events_log_file.write(event_confirmation_format(**kwargs))

    def log_confirmation(self, payload, commerce_id):
        format_params = {'commerce_id': commerce_id}
        format_params.update(**payload.data)
        with closing(self.journal_log_file) as journal_log_file:
            journal_log_file.write(log_confirmation_format(**format_params))

    @property
    def events_log_file(self):
        return self.log_file(EVENTS_LOG_FILE_NAME_FORMAT, EVENTS_LOG_FILE_DATE_FORMAT)

    @property
    def journal_log_file(self):
        return self.log_file(JOURNAL_LOG_FILE_NAME_FORMAT, JOURNAL_LOG_FILE_DATE_FORMAT)

    def log_file(self, log_file_name_format, log_file_date_format):
        santiago = pytz.timezone('America/Santiago')
        now = santiago.localize(datetime.datetime.now())
        file_name = log_file_name_format % now.strftime(log_file_date_format)
        return open(os.path.join(self.path, file_name), 'a+')


PAYMENT_FORMAT = (
    "          ;{pid:>12};   ;Filtro    ;Inicio                                  ;{date:<14};{time:<6};{request_ip:<15};OK ;                    ;Inicio de filtrado\n"  # noqa
    "          ;{pid:>12};   ;Filtro    ;tbk_param.txt                           ;{date:<14};{time:<6};{request_ip:<15};OK ;                    ;Archivo parseado\n"  # noqa
    "          ;{pid:>12};   ;Filtro    ;Terminado                               ;{date:<14};{time:<6};{request_ip:<15};OK ;                    ;Datos Filtrados con exito\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;inicio                                  ;{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Parseo realizado\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Datos en datos/tbk_config.dat\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Mac generado\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Construccion TBK_PARAM\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};TBK_PARAM encriptado\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Datos listos para ser enviados\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Medio 1: Transaccion segura\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Datos validados\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Token={token:}\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Redireccion web\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;pago      ;{webpay_server:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Todo OK\n"  # noqa
)

CONFIRMATION_FORMAT = (
    "          ;{pid:>12};   ;resultado ;Desencriptando                          ;{date:<14};{time:<6};{request_ip:<15};OK ;                    ;TBK_PARAM desencriptado\n"  # noqa
    "          ;{pid:>12};   ;resultado ;Validacion                              ;{date:<14};{time:<6};{request_ip:<15};OK ;                    ;Entidad emisora de los datos validada\n"  # noqa
    "          ;{pid:>12};   ;resultado ;{order_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;                    ;Parseo de los datos\n"  # noqa
    "          ;{pid:>12};   ;resultado ;{order_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;                    ;http://127.0.0.1/webpay/notify\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;transacc  ;{transaction_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};conectandose al port :(80)\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;resultado ;logro abrir_conexion                    ;{date:<14};{time:<6};{request_ip:<15}; 0 ;{commerce_id:<20};Abrio socket para conex-com\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;transacc  ;{transaction_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};POST a url http://127.0.0.1/webpay/notify\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;transacc  ;{transaction_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};mensaje enviado\n"  # noqa
    "          ;{pid:>12};   ;check_mac ;                                        ;{date:<14};{time:<6};EMPTY          ;OK ;                    ;Todo OK\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;transacc  ;{transaction_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Llego ACK del Comercio\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;resultado ;{order_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};tienda acepto transaccion\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;resultado ;{order_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};respuesta enviada a TBK (ACK)\n"  # noqa
    "{transaction_id:<10};{pid:>12};   ;resultado ;{order_id:<40};{date:<14};{time:<6};{request_ip:<15};OK ;{commerce_id:<20};Todo OK\n"  # noqa
)

JOURNAL_FORMAT = (
    "ACK; "
    "TBK_ORDEN_COMPRA=%(TBK_ORDEN_COMPRA)s; "
    "TBK_CODIGO_COMERCIO=%(commerce_id)s; "
    "TBK_TIPO_TRANSACCION=%(TBK_TIPO_TRANSACCION)s; "
    "TBK_RESPUESTA=%(TBK_RESPUESTA)s; "
    "TBK_MONTO=%(TBK_MONTO)s; "
    "TBK_CODIGO_AUTORIZACION=%(TBK_CODIGO_AUTORIZACION)s; "
    "TBK_FINAL_NUMERO_TARJETA=%(TBK_FINAL_NUMERO_TARJETA)s; "
    "TBK_FECHA_CONTABLE=%(TBK_FECHA_CONTABLE)s; "
    "TBK_FECHA_TRANSACCION=%(TBK_FECHA_TRANSACCION)s; "
    "TBK_HORA_TRANSACCION=%(TBK_HORA_TRANSACCION)s; "
    "TBK_ID_SESION=%(TBK_ID_SESION)s; "
    "TBK_ID_TRANSACCION=%(TBK_ID_TRANSACCION)s; "
    "TBK_TIPO_PAGO=%(TBK_TIPO_PAGO)s; "
    "TBK_NUMERO_CUOTAS=%(TBK_NUMERO_CUOTAS)s; "
    "TBK_VCI=%(TBK_VCI)s; "
    "TBK_MAC=%(TBK_MAC)s\n"
)
